package com.sun.xml.internal.stream.buffer.stax;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Collections;
import java.util.HashSet;
import java.io.IOException;
import com.sun.xml.internal.org.jvnet.staxex.Base64Data;
import com.sun.xml.internal.org.jvnet.staxex.XMLStreamWriterEx;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import com.sun.xml.internal.stream.buffer.XMLStreamBuffer;
import com.sun.xml.internal.stream.buffer.AbstractProcessor;

public class StreamWriterBufferProcessor extends AbstractProcessor
{
    public StreamWriterBufferProcessor() {
    }
    
    @Deprecated
    public StreamWriterBufferProcessor(XMLStreamBuffer buffer) {
        this.setXMLStreamBuffer(buffer, buffer.isFragment());
    }
    
    public StreamWriterBufferProcessor(XMLStreamBuffer buffer, boolean produceFragmentEvent) {
        this.setXMLStreamBuffer(buffer, produceFragmentEvent);
    }
    
    public final void process(XMLStreamBuffer buffer, XMLStreamWriter writer) throws XMLStreamException {
        this.setXMLStreamBuffer(buffer, buffer.isFragment());
        this.process(writer);
    }
    
    public void process(XMLStreamWriter writer) throws XMLStreamException {
        if (this._fragmentMode) {
            this.writeFragment(writer);
        }
        else {
            this.write(writer);
        }
    }
    
    @Deprecated
    public void setXMLStreamBuffer(XMLStreamBuffer buffer) {
        this.setBuffer(buffer);
    }
    
    public void setXMLStreamBuffer(XMLStreamBuffer buffer, boolean produceFragmentEvent) {
        this.setBuffer(buffer, produceFragmentEvent);
    }
    
    public void write(XMLStreamWriter writer) throws XMLStreamException {
        if (!this._fragmentMode) {
            if (this._treeCount > 1) {
                throw new IllegalStateException("forest cannot be written as a full infoset");
            }
            writer.writeStartDocument();
        }
        while (true) {
            int item = getEIIState(this.peekStructure());
            writer.flush();
            switch (item) {
                case 1: {
                    this.readStructure();
                    continue;
                }
                case 3:
                case 4:
                case 5:
                case 6: {
                    this.writeFragment(writer);
                    continue;
                }
                case 12: {
                    this.readStructure();
                    int length = this.readStructure();
                    int start = this.readContentCharactersBuffer(length);
                    String comment = new String(this._contentCharactersBuffer, start, length);
                    writer.writeComment(comment);
                    continue;
                }
                case 13: {
                    this.readStructure();
                    int length = this.readStructure16();
                    int start = this.readContentCharactersBuffer(length);
                    String comment = new String(this._contentCharactersBuffer, start, length);
                    writer.writeComment(comment);
                    continue;
                }
                case 14: {
                    this.readStructure();
                    char[] ch = this.readContentCharactersCopy();
                    writer.writeComment(new String(ch));
                    continue;
                }
                case 16: {
                    this.readStructure();
                    writer.writeProcessingInstruction(this.readStructureString(), this.readStructureString());
                    continue;
                }
                case 17: {
                    this.readStructure();
                    writer.writeEndDocument();
                    return;
                }
                default: {
                    throw new XMLStreamException("Invalid State " + item);
                }
            }
        }
    }
    
    public void writeFragment(XMLStreamWriter writer) throws XMLStreamException {
        if ((writer instanceof XMLStreamWriterEx)) {
            this.writeFragmentEx((XMLStreamWriterEx)writer);
        }
        else {
            this.writeFragmentNoEx(writer);
        }
    }
    
    public void writeFragmentEx(XMLStreamWriterEx writer) throws XMLStreamException {
        int depth = 0;
        int item = getEIIState(this.peekStructure());
        if (item == 1) {
            this.readStructure();
        }
        do {
            item = this.readEiiState();
            switch (item) {
                case 1: {
                    throw new AssertionError();
                }
                case 3: {
                    ++depth;
                    String uri = this.readStructureString();
                    String localName = this.readStructureString();
                    String prefix = this.getPrefixFromQName(this.readStructureString());
                    writer.writeStartElement(prefix, localName, uri);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 4: {
                    ++depth;
                    String prefix2 = this.readStructureString();
                    String uri2 = this.readStructureString();
                    String localName2 = this.readStructureString();
                    writer.writeStartElement(prefix2, localName2, uri2);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 5: {
                    ++depth;
                    String uri = this.readStructureString();
                    String localName = this.readStructureString();
                    writer.writeStartElement("", localName, uri);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 6: {
                    ++depth;
                    String localName3 = this.readStructureString();
                    writer.writeStartElement(localName3);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 7: {
                    int length = this.readStructure();
                    int start = this.readContentCharactersBuffer(length);
                    writer.writeCharacters(this._contentCharactersBuffer, start, length);
                    continue;
                }
                case 8: {
                    int length = this.readStructure16();
                    int start = this.readContentCharactersBuffer(length);
                    writer.writeCharacters(this._contentCharactersBuffer, start, length);
                    continue;
                }
                case 9: {
                    char[] c = this.readContentCharactersCopy();
                    writer.writeCharacters(c, 0, c.length);
                    continue;
                }
                case 10: {
                    String s = this.readContentString();
                    writer.writeCharacters(s);
                    continue;
                }
                case 11: {
                    CharSequence c2 = (CharSequence)this.readContentObject();
                    writer.writePCDATA(c2);
                    continue;
                }
                case 12: {
                    int length = this.readStructure();
                    int start = this.readContentCharactersBuffer(length);
                    String comment = new String(this._contentCharactersBuffer, start, length);
                    writer.writeComment(comment);
                    continue;
                }
                case 13: {
                    int length = this.readStructure16();
                    int start = this.readContentCharactersBuffer(length);
                    String comment = new String(this._contentCharactersBuffer, start, length);
                    writer.writeComment(comment);
                    continue;
                }
                case 14: {
                    char[] ch = this.readContentCharactersCopy();
                    writer.writeComment(new String(ch));
                    continue;
                }
                case 16: {
                    writer.writeProcessingInstruction(this.readStructureString(), this.readStructureString());
                    continue;
                }
                case 17: {
                    writer.writeEndElement();
                    if (--depth == 0) {
                        --this._treeCount;
                        continue;
                    }
                    continue;
                }
                default: {
                    throw new XMLStreamException("Invalid State " + item);
                }
            }
        } while (depth > 0 || this._treeCount > 0);
    }
    
    public void writeFragmentNoEx(XMLStreamWriter writer) throws XMLStreamException {
        int depth = 0;
        int item = getEIIState(this.peekStructure());
        if (item == 1) {
            this.readStructure();
        }
        do {
            item = this.readEiiState();
            switch (item) {
                case 1: {
                    throw new AssertionError();
                }
                case 3: {
                    ++depth;
                    String uri = this.readStructureString();
                    String localName = this.readStructureString();
                    String prefix = this.getPrefixFromQName(this.readStructureString());
                    writer.writeStartElement(prefix, localName, uri);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 4: {
                    ++depth;
                    String prefix2 = this.readStructureString();
                    String uri2 = this.readStructureString();
                    String localName2 = this.readStructureString();
                    writer.writeStartElement(prefix2, localName2, uri2);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 5: {
                    ++depth;
                    String uri = this.readStructureString();
                    String localName = this.readStructureString();
                    writer.writeStartElement("", localName, uri);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 6: {
                    ++depth;
                    String localName3 = this.readStructureString();
                    writer.writeStartElement(localName3);
                    this.writeAttributes(writer, this.isInscope(depth));
                    continue;
                }
                case 7: {
                    int length = this.readStructure();
                    int start = this.readContentCharactersBuffer(length);
                    writer.writeCharacters(this._contentCharactersBuffer, start, length);
                    continue;
                }
                case 8: {
                    int length = this.readStructure16();
                    int start = this.readContentCharactersBuffer(length);
                    writer.writeCharacters(this._contentCharactersBuffer, start, length);
                    continue;
                }
                case 9: {
                    char[] c = this.readContentCharactersCopy();
                    writer.writeCharacters(c, 0, c.length);
                    continue;
                }
                case 10: {
                    String s = this.readContentString();
                    writer.writeCharacters(s);
                    continue;
                }
                case 11: {
                    CharSequence c2 = (CharSequence)this.readContentObject();
                    if ((c2 instanceof Base64Data)) {
                        try {
                            Base64Data bd = (Base64Data)c2;
                            bd.writeTo(writer);
                            continue;
                        }
                        catch (IOException e) {
                            throw new XMLStreamException(e);
                        }
                    }
                    else {
                        writer.writeCharacters(c2.toString());
                        continue;
                    }
                }
                case 12: {
                    int length = this.readStructure();
                    int start = this.readContentCharactersBuffer(length);
                    String comment = new String(this._contentCharactersBuffer, start, length);
                    writer.writeComment(comment);
                    continue;
                }
                case 13: {
                    int length = this.readStructure16();
                    int start = this.readContentCharactersBuffer(length);
                    String comment = new String(this._contentCharactersBuffer, start, length);
                    writer.writeComment(comment);
                    continue;
                }
                case 14: {
                    char[] ch = this.readContentCharactersCopy();
                    writer.writeComment(new String(ch));
                    continue;
                }
                case 16: {
                    writer.writeProcessingInstruction(this.readStructureString(), this.readStructureString());
                    continue;
                }
                case 17: {
                    writer.writeEndElement();
                    if (--depth == 0) {
                        --this._treeCount;
                        continue;
                    }
                    continue;
                }
                default: {
                    throw new XMLStreamException("Invalid State " + item);
                }
            }
        } while (depth > 0 || this._treeCount > 0);
    }
    
    private boolean isInscope(int depth) {
        return this._buffer.getInscopeNamespaces().size() > 0 && depth == 1;
    }
    
    private void writeAttributes(XMLStreamWriter writer, boolean inscope) throws XMLStreamException {
        Set<String> prefixSet = inscope ? new HashSet() : Collections.emptySet();
        int item = this.peekStructure();
        if ((item & 0xF0) == 0x40) {
            item = this.writeNamespaceAttributes(item, writer, inscope, prefixSet);
        }
        if (inscope) {
            this.writeInscopeNamespaces(writer, prefixSet);
        }
        if ((item & 0xF0) == 0x30) {
            this.writeAttributes(item, writer);
        }
    }
    
    private static String fixNull(String s) {
        if (s == null) {
            return "";
        }
        return s;
    }
    
    private void writeInscopeNamespaces(XMLStreamWriter writer, Set<String> prefixSet) throws XMLStreamException {
        for (Map.Entry<String, String> e : this._buffer.getInscopeNamespaces().entrySet()) {
            String key = fixNull((String)e.getKey());
            if (!prefixSet.contains(key)) {
                writer.writeNamespace(key, (String)e.getValue());
            }
            else {
                continue;
            }
        }
    }
    
    private int writeNamespaceAttributes(int item, XMLStreamWriter writer, boolean collectPrefixes, Set<String> prefixSet) throws XMLStreamException {
        do {
            switch (getNIIState(item)) {
                case 1: {
                    writer.writeDefaultNamespace("");
                    if (collectPrefixes) {
                        prefixSet.add("");
                        break;
                    }
                    break;
                }
                case 2: {
                    String prefix = this.readStructureString();
                    writer.writeNamespace(prefix, "");
                    if (collectPrefixes) {
                        prefixSet.add(prefix);
                        break;
                    }
                    break;
                }
                case 3: {
                    String prefix = this.readStructureString();
                    writer.writeNamespace(prefix, this.readStructureString());
                    if (collectPrefixes) {
                        prefixSet.add(prefix);
                        break;
                    }
                    break;
                }
                case 4: {
                    writer.writeDefaultNamespace(this.readStructureString());
                    if (collectPrefixes) {
                        prefixSet.add("");
                        break;
                    }
                    break;
                }
            }
            this.readStructure();
            item = this.peekStructure();
        } while ((item & 0xF0) == 0x40);
        return item;
    }
    
    private void writeAttributes(int item, XMLStreamWriter writer) throws XMLStreamException {
        do {
            switch (getAIIState(item)) {
                case 1: {
                    String uri = this.readStructureString();
                    String localName = this.readStructureString();
                    String prefix = this.getPrefixFromQName(this.readStructureString());
                    writer.writeAttribute(prefix, uri, localName, this.readContentString());
                    break;
                }
                case 2: {
                    writer.writeAttribute(this.readStructureString(), this.readStructureString(), this.readStructureString(), this.readContentString());
                    break;
                }
                case 3: {
                    writer.writeAttribute(this.readStructureString(), this.readStructureString(), this.readContentString());
                    break;
                }
                case 4: {
                    writer.writeAttribute(this.readStructureString(), this.readContentString());
                    break;
                }
            }
            this.readStructureString();
            this.readStructure();
            item = this.peekStructure();
        } while ((item & 0xF0) == 0x30);
    }
}
